import argparse
import runpy
from baserun import Baserun


def main():
    parser = argparse.ArgumentParser(description='Baserun CLI tool')
    parser.add_argument('--api-url', default='https://baserun.ai/api/v1', help='Baserun API URL')
    parser.add_argument('module', help='Python module to run')
    args = parser.parse_args()

    Baserun.init(api_url=args.api_url)

    try:
        runpy.run_module(args.module, run_name="__main__")
    finally:
        Baserun.flush()


if __name__ == "__main__":
    main()



from importlib import import_module
import time
from .helpers import BaserunProvider, BaserunStepType, BaserunType

BANNED_CONFIG_KEYS = ['api_base', 'api_key', 'headers', 'organization', 'messages', 'prompt']

def monkey_patch_openai(log):
    try:
        openai = import_module("openai")

        original_completion_create = openai.Completion.create
        original_completion_acreate = openai.Completion.acreate
        original_chatcompletion_create = openai.ChatCompletion.create
        original_chatcompletion_acreate = openai.ChatCompletion.acreate

        def patched_completion_create(*args, **kwargs):
            start_time = time.time()
            response = original_completion_create(*args, **kwargs)
            end_time = time.time()

            prompt = kwargs.get('prompt', "")
            config = {key: value for key, value in kwargs.items() if key not in BANNED_CONFIG_KEYS}
            usage = response["usage"]
            output = response["choices"][0]["text"]

            log_entry = {
                "stepType": BaserunStepType.AUTO_LLM.name.lower(),
                "type": BaserunType.COMPLETION.name.lower(),
                "provider": BaserunProvider.OPENAI.name.lower(),
                "config": config,
                "prompt": {"content": prompt},
                "output": output,
                "startTimestamp": start_time,
                "completionTimestamp": end_time,
                "usage": usage,
            }

            log(log_entry)

            return response

        async def patched_completion_acreate(*args, **kwargs):
            start_time = time.time()
            response = await original_completion_acreate(*args, **kwargs)
            end_time = time.time()

            prompt = kwargs.get('prompt', "")
            config = {key: value for key, value in kwargs.items() if key not in BANNED_CONFIG_KEYS}
            usage = response["usage"]
            output = response["choices"][0]["text"]

            log_entry = {
                "stepType": BaserunStepType.AUTO_LLM.name.lower(),
                "type": BaserunType.COMPLETION.name.lower(),
                "provider": BaserunProvider.OPENAI.name.lower(),
                "config": config,
                "prompt": {"content": prompt},
                "output": output,
                "startTimestamp": start_time,
                "completionTimestamp": end_time,
                "usage": usage,
            }

            log(log_entry)

            return response

        def patched_chatcompletion_create(*args, **kwargs):
            start_time = time.time()
            response = original_chatcompletion_create(*args, **kwargs)
            end_time = time.time()

            messages = kwargs.get('messages', [])
            config = {key: value for key, value in kwargs.items() if key not in BANNED_CONFIG_KEYS}
            output = response["choices"][0]["message"]
            usage = response["usage"]

            log_entry = {
                "stepType": BaserunStepType.AUTO_LLM.name.lower(),
                "type": BaserunType.CHAT.name.lower(),
                "provider": BaserunProvider.OPENAI.name.lower(),
                "config": config,
                "messages": messages,
                "output": output,
                "startTimestamp": start_time,
                "completionTimestamp": end_time,
                "usage": usage,
            }

            log(log_entry)

            return response

        async def patched_chatcompletion_acreate(*args, **kwargs):
            start_time = time.time()
            response = await original_chatcompletion_acreate(*args, **kwargs)
            end_time = time.time()

            messages = kwargs.get('messages', [])
            config = {key: value for key, value in kwargs.items() if key not in BANNED_CONFIG_KEYS}
            output = response["choices"][0]["message"]
            usage = response["usage"]

            log_entry = {
                "stepType": BaserunStepType.AUTO_LLM.name.lower(),
                "type": BaserunType.CHAT.name.lower(),
                "provider": BaserunProvider.OPENAI.name.lower(),
                "config": config,
                "messages": messages,
                "output": output,
                "startTimestamp": start_time,
                "completionTimestamp": end_time,
                "usage": usage,
            }

            log(log_entry)

            return response

        openai.Completion.create = patched_completion_create
        openai.Completion.acreate = patched_completion_acreate
        openai.ChatCompletion.create = patched_chatcompletion_create
        openai.ChatCompletion.acreate = patched_chatcompletion_acreate

    except ModuleNotFoundError:
        return



import unittest
import baserun
from unittest.mock import MagicMock, patch


# Run using BASERUN_API_KEY="test-key" python3 -m unittest tests/unittests_test.py
class BaserunTests(baserun.BaserunTestCase):

    def setUp(self):
        self.mock_baserun_store = MagicMock()
        patcher = patch('baserun.Baserun.store_trace', new=self.mock_baserun_store)
        self.addCleanup(patcher.stop)
        patcher.start()

        self.next_assert = None

    def tearDown(self):
        if self.next_assert:
            self.next_assert()

    def test_log(self):
        log_name = "TestEvent"
        baserun.log(log_name, 'whatever')

        def assertion_func():
            stored_data = self.mock_baserun_store.call_args[0][0]
            self.assertIn(log_name, stored_data['steps'][0]['name'])
            self.assertIn('whatever', stored_data['steps'][0]['payload'])

        self.next_assert = assertion_func

    def test_log_with_payload(self):
        event_name = "TestEvent"
        event_payload = {
            "action": "called_api",
            "value": 42
        }

        baserun.log(event_name, payload=event_payload)

        def assertion_func():
            stored_data = self.mock_baserun_store.call_args[0][0]
            self.assertIn(event_name, stored_data['steps'][0]['name'])
            self.assertEqual(event_payload, stored_data['steps'][0]['payload'])

        self.next_assert = assertion_func

    def test_multiple_logs_same_baserun_id(self):
        log_name_1 = "TestEvent1"
        log_name_2 = "TestEvent2"
        baserun.log(log_name_1, 'string payload 1')
        baserun.log(log_name_2, 'string payload 2')

        def assertion_func():
            stored_data = self.mock_baserun_store.call_args[0][0]
            self.assertIn(log_name_1, stored_data['steps'][0]['name'])
            self.assertIn(log_name_2, stored_data['steps'][1]['name'])
            self.assertIn('string payload 1', stored_data['steps'][0]['payload'])
            self.assertIn('string payload 2', stored_data['steps'][1]['payload'])

        self.next_assert = assertion_func


if __name__ == '__main__':
    unittest.main()



import unittest
from unittest.mock import patch
import baserun
import os


# Run using python3 -m unittest tests/explicit_init.py
class TestBaserunInit(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        os.environ["BASERUN_API_KEY"] = "test-key"
        cls.api_key = "test_api_key"
        cls.api_url = "https://baserun.ai/api/v1"
        baserun.init(cls.api_url)

    @patch("baserun.flush")
    @patch("baserun.Baserun.store_trace")
    def test_explicit_log(self, mock_store_trace, mock_flush):

        @baserun.test
        def sample_test():
            baserun.log("TestEvent", "string payload")

        sample_test()
        baserun.flush()

        mock_store_trace.assert_called_once()
        stored_data = mock_store_trace.call_args[0][0]
        self.assertEqual(stored_data['steps'][0]['name'], "TestEvent")
        self.assertEqual(stored_data['steps'][0]['payload'], "string payload")

        mock_flush.assert_called_once()

    @patch("baserun.flush")
    @patch("baserun.Baserun.store_trace")
    def test_explicit_log_with_payload(self, mock_store_trace, mock_flush):
        log_name = "TestEvent"
        log_payload = {
            "action": "called_api",
            "value": 42
        }

        @baserun.test
        def sample_test():
            baserun.log(log_name, log_payload)

        sample_test()
        baserun.flush()

        mock_store_trace.assert_called_once()
        stored_data = mock_store_trace.call_args[0][0]
        self.assertEqual(stored_data['steps'][0]['name'], log_name)
        self.assertEqual(stored_data['steps'][0]['payload'], log_payload)

        mock_flush.assert_called_once()


if __name__ == '__main__':
    unittest.main()



from baserun import Baserun
import logging

logger = logging.getLogger(__name__)

run_url = None

def pytest_addoption(parser):
    parser.addoption("--baserun", action="store_true", help="Enable baserun functionality")
    parser.addoption("--baserun-api-url", default="https://baserun.ai/api/v1", help="Baserun API URL")
    parser.addoption("--no-flush", action="store_true", default=False,
                     help="do not flush to baserun, even if --baserun is used")


def pytest_sessionstart(session):
    if session.config.getoption("--baserun"):
        api_url = session.config.getoption("--baserun-api-url")
        Baserun.init(api_url)


def pytest_sessionfinish(session):
    global run_url
    if session.config.getoption("--baserun"):
        if session.config.getoption("--no-flush"):
            logger.info("Baserun flush disabled by --no-flush option.")
            return

        run_url = Baserun.flush()


def pytest_collection_modifyitems(config, items):
    if config.getoption("--baserun"):
        for item in items:
            item.obj = Baserun.test(item.obj)


def pytest_terminal_summary(terminalreporter):
    global run_url
    if run_url:
        terminalreporter.write_sep("=", "Baserun summary", blue=True)
        terminalreporter.write_line(f"Test results available at: {run_url}")


import unittest
import os
from baserun import Baserun


class BaserunTestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        api_url = os.environ.get("BASERUN_API_URL", "https://baserun.ai/api/v1")
        Baserun.init(api_url)

    def run(self, result=None):
        test_method = getattr(self, self._testMethodName)
        decorated_test_method = Baserun.test(test_method)
        setattr(self, self._testMethodName, decorated_test_method)
        super().run(result)

    @classmethod
    def tearDownClass(cls):
        Baserun.flush()



import baserun
from baserun import baserun as baserun_module


# Run using BASERUN_API_KEY="test-key" pytest --baserun --no-flush tests/pytest_test.py
def test_log_message():
    log_name = "TestEvent"
    baserun.log(log_name, "whatever")

    stored_data = baserun_module._thread_local.buffer
    assert stored_data[0]['stepType'] == 'log'
    assert log_name in stored_data[0]['name']
    assert "whatever" in stored_data[0]['payload']


def test_multiple_logs_same_baserun_id():
    log_name_1 = "TestEvent1"
    log_name_2 = "TestEvent2"

    baserun.log(log_name_1, 'string payload 1')
    baserun.log(log_name_2, 'string payload 2')

    stored_data = baserun_module._thread_local.buffer
    assert stored_data[0]['stepType'] == 'log'
    assert stored_data[1]['stepType'] == 'log'
    assert log_name_1 in stored_data[0]['name']
    assert log_name_2 in stored_data[1]['name']
    assert 'string payload 1' in stored_data[0]['payload']
    assert 'string payload 2' in stored_data[1]['payload']


def test_log_with_payload():
    event_name = "TestEvent"
    event_payload = {
        "action": "called_api",
        "value": 42
    }

    baserun.log(event_name, payload=event_payload)

    stored_data = baserun_module._thread_local.buffer
    assert stored_data[0]['stepType'] == 'log'
    assert stored_data[0]['name'] == event_name
    assert stored_data[0]['payload'] == event_payload



from enum import Enum, auto


class BaserunProvider(Enum):
    GOOGLE = auto()
    OPENAI = auto()
    LLAMA = auto()


class BaserunType(Enum):
    CHAT = auto()
    COMPLETION = auto()


class BaserunStepType(Enum):
    LOG = auto()
    AUTO_LLM = auto()
    CUSTOM_LLM = auto()


class TraceType(Enum):
    TEST = auto()
    PRODUCTION = auto()



import inspect
import json
import uuid
import os
import requests
import time
import threading
from typing import Callable, Dict, Optional, Union
from urllib.parse import urlparse
import warnings
from .helpers import BaserunStepType, TraceType
from .openai import monkey_patch_openai


class Baserun:
    _initialized = False
    _trace_id = None
    _traces = []
    _buffer = []
    _buffer_lock = threading.Lock()
    _api_base_url = None
    _api_key = None

    @staticmethod
    def init(api_base_url: str = "https://baserun.ai/api/v1") -> None:
        api_key = os.environ.get('BASERUN_API_KEY')
        if not api_key:
            raise ValueError("Baserun API key is missing. Ensure the BASERUN_API_KEY environment variable is set.")

        if Baserun._initialized:
            warnings.warn("Baserun has already been initialized. Additional calls to init will be ignored.")
            return

        Baserun._api_base_url = api_base_url
        Baserun._api_key = api_key
        Baserun._initialized = True

        monkey_patch_openai(Baserun._append_to_buffer)

    @staticmethod
    def _trace(func: Callable, trace_type: TraceType, metadata: Optional[Dict] = None):
        if inspect.iscoroutinefunction(func):
            async def wrapper(*args, **kwargs):
                if not Baserun._initialized or Baserun._trace_id:
                    return await func(*args, **kwargs)

                test_name = func.__name__
                test_inputs = []
                for input_name, input_value in kwargs.items():
                    if inspect.iscoroutine(input_value):
                        input_result = input_value.__name__
                    else:
                        input_result = input_value
                    test_inputs.append(f"{input_name}: {input_result}")

                test_execution_id = str(uuid.uuid4())
                Baserun._trace_id = test_execution_id
                Baserun._buffer = []
                start_time = time.time()

                try:
                    result = await func(*args, **kwargs)
                    end_time = time.time()
                    Baserun.store_trace({
                        'type': trace_type,
                        'testName': test_name,
                        'testInputs': test_inputs,
                        'id': test_execution_id,
                        'result': str(result) if result is not None else '',
                        'startTimestamp': start_time,
                        'completionTimestamp': end_time,
                        "steps": Baserun._buffer,
                        "metadata": metadata,
                    })
                    return result
                except Exception as e:
                    end_time = time.time()
                    Baserun.store_trace({
                        'type': trace_type,
                        'testName': test_name,
                        'testInputs': test_inputs,
                        'id': test_execution_id,
                        'error': str(e),
                        'startTimestamp': start_time,
                        'completionTimestamp': end_time,
                        "steps": Baserun._buffer,
                        "metadata": metadata,
                    })
                    raise e
                finally:
                    if trace_type == TraceType.PRODUCTION:
                        Baserun.flush()

                    Baserun._buffer = []
                    Baserun._trace_id = None
        else:
            def wrapper(*args, **kwargs):
                if not Baserun._initialized or Baserun._trace_id:
                    return func(*args, **kwargs)

                test_name = func.__name__
                test_inputs = []
                for input_name, input_value in kwargs.items():
                    test_inputs.append(f"{input_name}: {input_value}")

                test_execution_id = str(uuid.uuid4())
                Baserun._trace_id = test_execution_id
                Baserun._buffer = []
                start_time = time.time()

                try:
                    result = func(*args, **kwargs)
                    end_time = time.time()
                    Baserun.store_trace({
                        'type': trace_type,
                        'testName': test_name,
                        'testInputs': test_inputs,
                        'id': test_execution_id,
                        'result': str(result) if result is not None else '',
                        'startTimestamp': start_time,
                        'completionTimestamp': end_time,
                        "steps": Baserun._buffer,
                        "metadata": metadata,
                    })
                    return result
                except Exception as e:
                    end_time = time.time()
                    Baserun.store_trace({
                        'type': trace_type,
                        'testName': test_name,
                        'testInputs': test_inputs,
                        'id': test_execution_id,
                        'error': str(e),
                        'startTimestamp': start_time,
                        'completionTimestamp': end_time,
                        "steps": Baserun._buffer,
                        "metadata": metadata,
                    })
                    raise e
                finally:
                    if trace_type == TraceType.PRODUCTION:
                        Baserun.flush()

                    Baserun._buffer = []
                    Baserun._trace_id = None

        return wrapper

    @staticmethod
    def trace(func: Callable, metadata: Optional[Dict] = None):
        return Baserun._trace(func, TraceType.PRODUCTION, metadata)

    @staticmethod
    def test(func: Callable, metadata: Optional[Dict] = None):
        return Baserun._trace(func, TraceType.TEST, metadata)

    @staticmethod
    def log(name: str, payload: Union[str, Dict]):
        if not Baserun._initialized:
            return

        if not Baserun._trace_id:
            warnings.warn("baserun.log was called outside of a Baserun decorated trace. The log will be ignored.")
            return

        log_entry = {
            "stepType": BaserunStepType.LOG.name.lower(),
            "name": name,
            "payload": payload,
            "timestamp": time.time(),
        }

        Baserun._append_to_buffer(log_entry)

    @staticmethod
    def store_trace(trace_data: Dict):
        Baserun._traces.append(trace_data)

    @staticmethod
    def flush():
        if not Baserun._initialized:
            warnings.warn("Baserun has not been initialized. No data will be flushed.")
            return

        if not Baserun._traces:
            return

        headers = {"Authorization": f"Bearer {Baserun._api_key}"}

        try:
            if all(trace.get('type') == TraceType.TEST for trace in Baserun._traces):
                response = requests.post(f"{Baserun._api_base_url}/runs", json=json.loads(json.dumps({"testExecutions": Baserun._traces}, default=str)), headers=headers)
                response.raise_for_status()

                response_data = response.json()
                test_run_id = response_data['id']
                parsed_url = urlparse(Baserun._api_base_url)
                url = f"{parsed_url.scheme}://{parsed_url.netloc}/runs/{test_run_id}"
                return url
            elif all(trace.get('type') == TraceType.PRODUCTION for trace in Baserun._traces):
                response = requests.post(f"{Baserun._api_base_url}/traces", json=json.loads(json.dumps({"traces": Baserun._traces}, default=str)), headers=headers)
                response.raise_for_status()
                return
            else:
                warnings.warn("Inconsistent trace types, skipping Baserun upload")
        except requests.RequestException as e:
            warnings.warn(f"Failed to upload results to Baserun: {str(e)}")

        finally:
            Baserun._traces = []

    @staticmethod
    def _append_to_buffer(log_entry: Dict):
        with Baserun._buffer_lock:
            Baserun._buffer.append(log_entry)
